VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CFloodFill"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' *********************************************************************
'  Copyright 2001, Karl E. Peterson, All Rights Reserved.
'  http://www.mvps.org/vb
' *********************************************************************
'  You are free to use this code within your own applications, but you
'  are expressly forbidden from selling or otherwise distributing this
'  source code without prior written consent.
' *********************************************************************
Option Explicit

Private Declare Function ExtFloodFill Lib "gdi32" (ByVal hDC As Long, ByVal X As Long, ByVal Y As Long, ByVal crColor As Long, ByVal wFillType As Long) As Long
Private Declare Function GetSysColor Lib "user32" (ByVal nIndex As Long) As Long

' ExtFloodFill style flags
Public Enum FillModes
   FLOODFILLBORDER = 0
   FLOODFILLSURFACE = 1
End Enum

' Member variables
Private m_Canvas As Object
Private m_FillColor As Long
Private m_BorderColor As Long
Private m_FillMode As FillModes
Private m_FillStyle As FillStyleConstants

' **********************************************
'  Initialization / Termination
' **********************************************
Private Sub Class_Initialize()
   m_FillColor = CheckSysColor(vbWindowText)
   m_BorderColor = CheckSysColor(vbWindowBackground)
   m_FillMode = FLOODFILLSURFACE
   m_FillStyle = vbFSSolid
End Sub

' **********************************************
'  Public Properties
' **********************************************
Public Property Let BorderColor(ByVal NewVal As OLE_COLOR)
   ' Store bordercolor: used for surface
   ' fills, as well.
   m_BorderColor = CheckSysColor(NewVal)
End Property

Public Property Get BorderColor() As OLE_COLOR
   BorderColor = m_BorderColor
End Property

Public Property Set Canvas(ByVal NewObj As Object)
   ' This class is only designed to work
   ' with forms and picture controls.
   If Not TypeOf NewObj Is Form Then
      If Not TypeOf NewObj Is PictureBox Then
         ' Type mismatch error.
         Err.Raise Number:=13, Source:="CFloodFill.Canvas", _
            Description:="Canvas must be either a Form or PictureBox."
         Exit Property
      End If
   End If
   ' If we're still here, store ref.
   Set m_Canvas = NewObj
End Property

Public Property Get Canvas() As Object
   Set Canvas = m_Canvas
End Property

Public Property Let FillColor(ByVal NewVal As OLE_COLOR)
   ' Store fillcolor
   m_FillColor = CheckSysColor(NewVal)
End Property

Public Property Get FillColor() As OLE_COLOR
   FillColor = m_FillColor
End Property

Public Property Let FillMode(ByVal NewVal As FillModes)
   ' FLOODFILLBORDER: The fill area is bounded
   ' by the color specified by the BorderColor
   ' property. This style is identical to the
   ' filling performed by the FloodFill function.
   ' FLOODFILLSURFACE: The fill area is defined
   ' by the color that is specified by BorderColor.
   ' Filling continues outward in all directions
   ' as long as the color is encountered. This
   ' style is useful for filling areas with
   ' multicolored boundaries.
   m_FillMode = NewVal
End Property

Public Property Get FillMode() As FillModes
   FillMode = m_FillMode
End Property

Public Property Let FillStyle(ByVal NewVal As FillStyleConstants)
   m_FillStyle = NewVal
End Property

Public Property Get FillStyle() As FillStyleConstants
   FillStyle = m_FillStyle
End Property

' **********************************************
'  Public Methods
' **********************************************
Public Sub Paint(ByVal X As Single, ByVal Y As Single)
   Dim xP As Long, yP As Long
   Dim oldFillColor As Long
   Dim oldFillStyle As Long
   
   ' Gotta have an m_Canvas!
   If m_Canvas Is Nothing Then Exit Sub
   
   ' Convert coordinates to pixels.
   xP = m_Canvas.ScaleX(X, m_Canvas.ScaleMode, vbPixels)
   yP = m_Canvas.ScaleY(Y, m_Canvas.ScaleMode, vbPixels)
   
   ' Cache current fill attributes, set new.
   oldFillColor = m_Canvas.FillColor
   oldFillStyle = m_Canvas.FillStyle
   m_Canvas.FillColor = m_FillColor
   m_Canvas.FillStyle = m_FillStyle
   
   ' Flood it!
   Call ExtFloodFill(m_Canvas.hDC, xP, yP, m_BorderColor, m_FillMode)
   
   ' Restore cached fill attributes.
   m_Canvas.FillColor = oldFillColor
   m_Canvas.FillStyle = oldFillStyle
End Sub

' **********************************************
'  Private Methods
' **********************************************
Private Function CheckSysColor(ByVal Color As Long) As Long
   Const HighBit As Long = &H80000000
   
   ' If high bit set, strip, and get system color.
   If Color And HighBit Then
      CheckSysColor = GetSysColor(Color And Not HighBit)
   Else
      CheckSysColor = Color
   End If
End Function
